RecvError

Enum RecvError 

pub enum RecvError {
    Timeout,
    Disconnected,
    IO(Error),
}
The result of waiting for an update from an EventSource with a timeout.

This is quite similar to the standard library mpsc::RecvTimeoutError, but also permitting an io::Error which may result from reading from standard input.

Variants



Timeout

No event was received because we timed out.



Disconnected

The source is disconnected and there are no more messages.



IO(Error)

An IO error occurred while trying to read an event.