Selection

Struct Selection 

pub struct Selection<'a, T: Send + Sync + 'static> { /* private fields */ }
The selected items when the picker quits.

This is the return type of the various pick_multi* methods of a Picker. Iterate over the picked items with iter. Also see the documentation for multiple selections

The lifetime of this struct is bound to the lifetime of the picker from which it originated.

Implementations

Source

impl<'a, T: Send + Sync + 'static> Selection<'a, T>

pub fn iter(&self) -> impl ExactSizeIterator<Item = &'a T> + DoubleEndedIterator

Returns an iterator over the other selected items.

The iterator contains each selected item exactly once, sorted by index based on the order in which the picker received the items. If multiple threads populate the picker, the relative order between different threads is unspecified. Note that items are deduplicated based on the selection index instead of using any properties of the type T itself.

See iter_selected_order to obtain the items in the order selected by the user.

The iterator will be empty if the picker quit without selecting any items.

pub fn iter_selected_order( &self, ) -> impl ExactSizeIterator<Item = &'a T> + DoubleEndedIterator

Returns an iterator over the selected items, sorted by selection order.

The iterator contains each selected item exactly once. If an item is un-selected and selected again, the order is determined by the final selection.

Note that the current implementation does not internally store the selected items by selection order, so calling this method requires allocating a new container and then sorting.

The iterator will be empty if the picker quit without selecting any items.

pub fn is_empty(&self) -> bool

Returns if there were no selected items.

pub fn len(&self) -> usize

Returns the number of selected items.