pub struct Observer<T> { /* private fields */ }Expand description
An Observer watching for a single message T.
This is similar to the ‘receiver’ end of a channel of length 1, but instead of blocking, the
‘sender’ always overwrites any element in the channel. In particular, any message obtained by
recv or try_recv is guaranteed to be the most
up-to-date at the moment when the message is received.
The channel may be updated when not observed. Receiving a message moves it out of the observer.
An observer can be cheaply cloned (a single Arc::clone) in order to watch for the message
simultaneously from different threads.
Implementations
Sourceimpl<T> Observer<T>
impl<T> Observer<T>
Sourcepub fn recv(&self) -> Result<T, RecvError>
pub fn recv(&self) -> Result<T, RecvError>
Receive a message, blocking until a message is available or the channel disconnects.
Sourcepub fn try_recv(&self) -> Result<T, TryRecvError>
pub fn try_recv(&self) -> Result<T, TryRecvError>
Optimistically receive a message if one is available without blocking the current thread.
This operation will fail if there is no message or if there are are no remaining senders.